;;;; srfi-27-uniform-random.scm
;;;; Kon Lovett, Feb '10

(module srfi-27-uniform-random

  (;export
    *make-uniform-random-integers
    make-uniform-random-integers
    make-uniform-random-reals)

  (import (except scheme + - * quotient = <)
          chicken
          (only numbers + - * quotient = <)
          (only miscmacros exchange!)
          random-source
          (only srfi-27 current-random-source)
          (only srfi-27-numbers
            check-integer check-positive-integer
            check-real-precision))

  (require-library
    miscmacros
    vector-lib numbers
    random-source srfi-27 srfi-27-numbers)

  (declare (not usual-integrations + - * quotient = <))

;;; Uniform random integers in [low high] by precision

(define (*make-uniform-random-integers low high prec rand)
  (let ((range (quotient (+ (- high low) 1) prec)))
      (cond ((and (= 0 low) (= 1 prec)) (lambda () (rand range)))
            ((= 0 low)                  (lambda () (* (rand range) prec)))
            (else                       (lambda () (+ low (* (rand range) prec))))) ) )

(define (make-uniform-random-integers #!key (high #f) (low 0) (precision 1)
                                            (source (current-random-source)))
  (check-random-source 'make-uniform-random-integers source 'source)
  (unless high (set! high (*random-source-maximum-range source) #;(- (*random-source-maximum-range source) 1)))
  (check-integer 'make-uniform-random-integers high 'high)
  (check-integer 'make-uniform-random-integers low 'low)
  (check-positive-integer 'make-uniform-random-integers precision 'precision)
  ; Handle swapped bounds (could be negative)
  (unless (< low high) (exchange! low high))
  ; Cannot have a 0 range
  (when (= low high)
    (error 'make-uniform-random-integers "`low' equals `high'" low high))
  ; Cannot have precision outside of range
  (when (< (- high low) precision)
    (error 'make-uniform-random-integers "`precision' greater-than range" precision low high))
  (values
    (*make-uniform-random-integers low high precision ((@random-source-make-integers source)))
    (lambda () (values high low precision source)) ) )

;;; Uniform random reals in (0.0 1.0) by precion

(define (make-uniform-random-reals #!key (precision #f) (source (current-random-source)))
  (check-random-source 'make-uniform-random-reals source 'source)
  (when precision (check-real-precision 'make-uniform-random-reals precision 'precision))
  (values
    ((@random-source-make-reals source) precision)
    (lambda () (values precision source)) ) )

) ;module srfi-27-uniform-random
